(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs/Rx'), require('immutable')) :
  typeof define === 'function' && define.amd ? define(['exports', 'rxjs/Rx', 'immutable'], factory) :
  (factory((global.rxstate = global.rxstate || {}),global.Rx,global.immutable));
}(this, (function (exports,Rx,immutable) { 'use strict';

Rx = 'default' in Rx ? Rx['default'] : Rx;

var createAction = function createAction() {
    var bus = new Rx.Subject();
    var creator = function creator(val) {
        return bus.next(val);
    };
    creator.$ = bus;
    return creator;
};

var defaultCombinator = function defaultCombinator(state, data) {
    return state.merge(data);
};

var createStore = function createStore(_ref) {
    var streams = _ref.streams,
        defaultState = _ref.defaultState,
        _ref$combinator = _ref.combinator,
        combinator = _ref$combinator === undefined ? defaultCombinator : _ref$combinator;

    // convert default state to immutable if needed
    var immutableState = immutable.fromJS(defaultState);
    // create result store stream
    var subj = new Rx.Subject();

    // create clean action
    var clear = createAction();
    clear.$.map(function (newData) {
        return newData ? immutable.fromJS(newData) : immutableState;
    }).subscribe(subj);
    // plug in user actions
    streams.map(function (s$) {
        return s$.map(function (val) {
            return immutable.fromJS(val);
        }).subscribe(subj);
    });

    // init result stream
    var store = new Rx.ReplaySubject(1);
    // start with default state
    subj.startWith(immutableState)
    // combine results
    .scan(combinator)
    // push result to final stream
    .subscribe(store);

    // append clear method
    store.clear = clear;

    return store;
};

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

















var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

// create status action
var createStatus = function createStatus() {
    var statusKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'status';

    var status = createAction();
    status.$ = status.$.map(function (s) {
        return immutable.fromJS(defineProperty({}, statusKey, s));
    });
    return status;
};

exports.createAction = createAction;
exports.createStore = createStore;
exports.createStatus = createStatus;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=component.js.map
